logmode +addin

%group1 = ""
%group2 = ""
!result = @uidialog("caption", "Canonical Correlations", "edit", %group1, "First group", "edit", %group2, "Second group")
if !result = -1 then
	stop
endif

'check groups are valid
if @isobject(%group1) = 0 then
	%msg = @upper(%group1) + " is not a valid group object."
	@uiprompt(%msg)
	stop	
endif
	
if @isobject(%group2) = 0 then
	%msg = @upper(%group2) + " is not a valid group object."
	@uiprompt(%msg)
	stop	
endif

%type = {%group1}.@type
if %type<>"GROUP" then
	%msg = @upper(%group1) + " is not a valid group object."
	@uiprompt(%msg)
	stop
endif
%type = {%group2}.@type
if %type<>"GROUP" then
	%msg = @upper(%group2) + " is not a valid group object."
	@uiprompt(%msg)
	stop
endif

%tn = "cancorr"
if @isobject(%tn) then
	%tn = @getnextname(%tn)
endif


table {%tn}
if {%group1}.@count >= {%group2}.@count then
	call cancorrtab({%tn},{%group1},{%group2})
else
	call cancorrtab({%tn},{%group2},{%group1})
endif
show {%tn}

subroutine local cancorrtab(table out, group zs, group xs)
	matrix zcoef
	matrix xcoef
	matrix zstdcoef
	matrix xstdcoef
	vector rho
	!usedobs = 0
	call cancorr(zs,xs,zcoef,xcoef,zstdcoef,xstdcoef,rho, !usedobs)
	matrix stats
	call cancorrstats(rho,stats)
	
	!cols = @recode(zs.@count>xs.@count,zs.@count,xs.@count)
	!cols = @recode(!cols<5,5,!cols)
	out(1,!cols) = ""   'trick to ensure the table has enough columns
	
	'table header
	!rowc = 1
	%title =  "Canonical Correlation Analysis"
	out(!rowc,1) =%title
	!rowc=!rowc+1
	%title = "Groups analysed: " + zs.@name + " and " + xs.@name	
	out(!rowc,1) = %title
	!rowc=!rowc+1
	%datestring = "Date: " + @datestr(@now,"MM/DD/YY") + "   Time: " + @datestr(@now,"HH:MI")
	out(!rowc,1) = %datestring
	!rowc=!rowc+1
	%samplestr = "Sample: " + @pagesmpl
	out(!rowc,1) = %samplestr
	!rowc=!rowc+1
	%obsstr = "Included observations: " + @str(!usedobs)
	out(!rowc,1) = %obsstr
	!rowc=!rowc+1
	out.setlines(!rowc) +d
	!rowc=!rowc+1
	
	'correlations
	out(!rowc,2) = "Canonical"
	out(!rowc,3) = "Squared"
	out(!rowc,4) = "Eigenvalue"
	out(!rowc,5) = "Likelihood"
	!rowc=!rowc+1
	out(!rowc,2) = "Correlation"
	out(!rowc,3) = "Can. Cor."
	out(!rowc,5) = "ratio"
	!rowc=!rowc+1
	out.setlines(!rowc) +d
	!rowc=!rowc+1
	for !i=1 to @rows(rho)
		out(!rowc,1) = !i
		out(!rowc,2) = stats(!i,1)
		out(!rowc,3) = stats(!i,2)
		out(!rowc,4) = stats(!i,3)
		out(!rowc,5) = stats(!i,4)
		!rowc=!rowc+1
	next
	out.setlines(!rowc) +d
	!rowc=!rowc+1	
	
	'statistics
	out(!rowc,1) = "Wilks' Lambda"
	out(!rowc,3) = stats(1,4)
	!rowc=!rowc+1
	out(!rowc,1) = "Pillai's Trace"
	out(!rowc,3) = stats(1,5)
	!rowc=!rowc+1
	out(!rowc,1) = "Hotelling-Lawley Trace"
	out(!rowc,3) = stats(1,6)
	!rowc=!rowc+1
	out(!rowc,1) = "Roy's Greatest Root"
	out(!rowc,3) = stats(1,3)
	!rowc=!rowc+1
	out.setlines(!rowc) +d
	!rowc=!rowc+1
	!rowc=!rowc+1
	
	'raw coefficients
	!min = @recode(xs.@count>zs.@count,zs.@count,xs.@count)
	
	%title = "Canonical Coefficients for " + xs.@name
	out(!rowc,1) = %title
	!rowc=!rowc+1
	for !i=1 to !min
		%head = xs.@name + @str(!i)
		out(!rowc,!i+1) = %head
	next
	!rowc=!rowc+1
	for !i=1 to xs.@count
		!j=!rowc+!i-1
		out(!j,1) = xs.@seriesname(!i)
	next
	for !i=1 to xs.@count
		for !j=1 to xs.@count
			!row = !rowc+!i-1
			!col = !j+1
			out(!row,!col) = xcoef(!i,!j)
		next
	next
	!rowc=!rowc+2+xs.@count
	
	%title = "Canonical Coefficients for " + zs.@name
	out(!rowc,1) = %title
	!rowc=!rowc+1
	for !i=1 to !min
		%head = zs.@name + @str(!i)
		out(!rowc,!i+1) = %head
	next
	!rowc=!rowc+1
	for !i=1 to zs.@count
		!j=!rowc+!i-1
		out(!j,1) = zs.@seriesname(!i)
	next
	for !i=1 to zs.@count
		for !j=1 to xs.@count
			!row = !rowc+!i-1
			!col = !j+1
			out(!row,!col) = zcoef(!i,!j)
		next
	next
	!rowc=!rowc+zs.@count
	out.setlines(!rowc) +d
	!rowc=!rowc+2	


	%title = "Standardised Canonical Coefficients for " + xs.@name
	out(!rowc,1) = %title
	!rowc=!rowc+1
	for !i=1 to !min
		%head = xs.@name + @str(!i)
		out(!rowc,!i+1) = %head
	next
	!rowc=!rowc+1
	for !i=1 to xs.@count
		!j=!rowc+!i-1
		out(!j,1) = xs.@seriesname(!i)
	next
	for !i=1 to xs.@count
		for !j=1 to xs.@count
			!row = !rowc+!i-1
			!col = !j+1
			out(!row,!col) = xstdcoef(!i,!j)
		next
	next
	!rowc=!rowc+2+xs.@count
	
	%title = "Standardised Canonical Coefficients for " + zs.@name
	out(!rowc,1) = %title
	!rowc=!rowc+1
	for !i=1 to !min
		%head = zs.@name + @str(!i)
		out(!rowc,!i+1) = %head
	next
	!rowc=!rowc+1
	for !i=1 to zs.@count
		!j=!rowc+!i-1
		out(!j,1) = zs.@seriesname(!i)
	next
	for !i=1 to zs.@count
		for !j=1 to xs.@count
			!row = !rowc+!i-1
			!col = !j+1
			out(!row,!col) = zstdcoef(!i,!j)
		next
	next
	!rowc=!rowc+zs.@count
	out.setlines(!rowc) +d
	!rowc=!rowc+1	

	
endsub


subroutine local cancorr(group zs, group xs, matrix ZCoef, matrix XCoef, matrix ZstdCoef, matrix XstdCoef, vector Rho, scalar !usedobs)
	local smpl
	series msmpl = 1
	group big zs xs
	smpl if @robs(big) = big.@count and msmpl=1
	!usedobs = @obssmpl
	sym bigcov = @covs(big)
	
	sym zscov = @covs(zs)
	sym xscov = @covs(xs)
	ZCoef = @subextract(bigcov,1,zs.@count+1,zs.@count,big.@count)
	
	'take cholesky factors
	matrix zscovchol = @cholesky(zscov)
	matrix xscovchol = @cholesky(xscov)
	
	'prescale
	ZCoef = @inverse(zscovchol )*ZCoef*@inverse(@transpose(xscovchol))

	'take svd
	ZCoef = @svd(ZCoef,rho,XCoef)
	
	'descale
	ZCoef = @inverse(@transpose(zscovchol))*ZCoef
	XCoef = @transpose(XCoef)*@inverse(xscovchol)
	XCoef = @transpose(XCoef)
	
	'de-standardise
	vector zstd = @getmaindiagonal(zscov)
	zstd = @sqrt(zstd)
	Zstdcoef = zcoef 'just to size it correctly
	for !i=1 to @columns(zcoef)
		vector temp = @rowextract(zcoef,!i)
		temp = temp*zstd(!i)
		rowplace(zstdcoef,temp,!i)
		d temp
	next

	vector xstd = @getmaindiagonal(xscov)
	xstd = @sqrt(xstd)
	xstdcoef = xcoef 'just to size it correctly
	for !i=1 to @columns(xcoef)
		vector temp = @rowextract(xcoef,!i)
		temp = temp*xstd(!i)
		rowplace(xstdcoef,temp,!i)
		d temp
	next

	
endsub

subroutine local cancorrstats(vector rho, matrix stats)
	!rows = @rows(rho)
	matrix(!rows,6) stats
	for !i=1 to !rows
		stats(!i,1) = rho(!i)  'can cor
		stats(!i,2) = rho(!i)^2 'can cor squared
		stats(!i,3) = (rho(!i)^2)/(1-rho(!i)^2)  'eigenvalue
		!lr = 1
		!pt = 0
		!hlt = 0
		for !j=!i to !rows
			!lr = !lr * (1-rho(!j)^2)		
			!pt = !pt + (1-rho(!j)^2)		
			!hlt = !hlt + rho(!j)^2/(1-rho(!j)^2)		
		next
		stats(!i,4) = !lr  'lr stat (Wilks' Lambda)
		stats(!i,5) = !pt  'Pillai's Trace
		stats(!i,6) = !hlt  'Hotelling-Lawley Trace
	next
endsub
